﻿TuiRaidTracker = AceLibrary("AceAddon-2.0"):new("AceDebug-2.0","AceEvent-2.0","AceHook-2.1","AceDB-2.0","FuBarPlugin-2.0")

local PC = AceLibrary("PaintChips-2.0")
local BC = AceLibrary("Babble-Class-2.2")
local Tablet = AceLibrary("Tablet-2.0")
local crayon = AceLibrary("Crayon-2.0")
local dewdrop = AceLibrary("Dewdrop-2.0")

TuiRaidTracker:RegisterDB("TuiRaidTrackerDB", "TuiRaidTrackerDBPerChar")
TuiRaidTracker.roster = AceLibrary("Rosterdaf-2.1")
TuiRaidTracker.clickableTooltip = true
TuiRaidTracker.hasIcon = true

TuiRaidTracker:RegisterDefaults('account', {
    settings = {
        minrarity = 3
    },
    tt = {
        online = true,
        loot = true,
        zone = true,
        kills = true
    }
})

TLR = {}

function TuiRaidTracker:OnEnable()
    --self:SetDebugging(true)
    self:Debug("Enabling TuiRaidTracker")
    if not TLR then TLR = {} end
    if not TLR.raid then TLR.raid = {} end
    
    self:RegisterBucketEvent("PARTY_MEMBERS_CHANGED",1,"ROSTER_UPDATE") -- PARTY SUPPORT !
    self:RegisterBucketEvent("RAID_ROSTER_UPDATE",1,"ROSTER_UPDATE")
    self:ROSTER_UPDATE();
end


function TuiRaidTracker:OnClick()
    self:Debug("OnClick");
    --self.ToggleTooltipDetached(TuiRaidTracker)
end

function TuiRaidTracker:CreateNewRaid(id)
    TLR.raid[id] = {
        info = { start = time() },
        players = {},
        online = {},
        loot = {},
        joins = {},
        leaves = {},
        kills = {},
        zones = {}
    }
end

function TuiRaidTracker:StartTracking()

    if self.tracking then
        error("already tracking :(");
        return
    end
    
	if not TLR.cr then
        self:Debug("start tracking (new)")
        TLR.cr = time()
        self:CreateNewRaid(TLR.cr);
    else
        self:Debug("start tracking (continue)")
    end

    self.cr = TLR.raid[TLR.cr]
    self.tracking = true

    -- check current raid / party
    for unit in self.roster:IterateRoster(false) do
        self:Debug("startscan",unit.name,unit.unitid)
        self:ChangeOnlinestatus(unit.name,unit.online)
    end
    
    -- check current zone
    self:ZONE_CHANGED_NEW_AREA()

    -- register events for tracking
	self:RegisterEvent("RosterLib-daf_UnitChanged","RosterLib_UnitChanged")
    self:RegisterEvent("CHAT_MSG_LOOT") -- for logging loot
    self:RegisterEvent("ZONE_CHANGED_NEW_AREA") -- for logging zones
    self:RegisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH") -- for logging boss kills
    self.trackingevents = true
    self:UpdateDisplay()
end

function TuiRaidTracker:StopTracking()
    self:Debug("StopTracking")
	local id = TLR.cr
	if id then
		TLR.raid[TLR.cr].info.ende = time()
		TLR.cr = nil
		self.cr = nil
	end
	self.tracking = false
	self:UpdateDisplay()

    if self.trackingevents then
        self:UnregisterEvent("CHAT_MSG_LOOT")
        self:UnregisterEvent("RosterLib-daf_UnitChanged")
        self:UnregisterEvent("ZONE_CHANGED_NEW_AREA")
        self:UnregisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH")
        self.trackingevents = false
    end
end

function TuiRaidTracker:ROSTER_UPDATE() -- this is just to enable disable tracking
    local num = GetNumRaidMembers()
    if num == 0 then num = GetNumPartyMembers() end -- PARTY SUPPORT !
  
    if num == 0 then
        self:StopTracking()
    else
        if not self.tracking then
            self:StartTracking()
        end
    end
end

function TuiRaidTracker:FormatDate(timestamp)
    -- TODO localisation
    return date("%x %X", timestamp)
end

function TuiRaidTracker:OnTextUpdate()
    self:SetText(string.format("TuiRaidTracker"))
end

function TuiRaidTracker:OnTooltipUpdate()
    
    local r,g,b
    
    if not self.tracking then
        local c = Tablet:AddCategory(
            'columns', 1,
            'text', 'TuiRaidTracker Status'
        )
        
        c:AddLine(
            'text','Tracking not active.'
        )
    end
 
    if not self.cr then return end
    
    local c_online = Tablet:AddCategory(
        'columns', 1,
        'hasCheck', true,
        'checked', self.db.account.tt.online,
        'showWithoutChildren', true,
        'text', 'Online',
        'func', function()
            if self.db.account.tt.online then
                self.db.account.tt.online = false
            else
                self.db.account.tt.online = true
            end
        end,
        'arg1', self    
    )
    
    if self.db.account.tt.online then
        local online = ""
        local offline = ""
        
        for name,on in pairs(self.cr.online) do
            if on == 1 then 
                online = string.format("%s %s",online,name)
            else
                offline = string.format("%s %s",offline,name)
            end
        end
    
         c_online:AddLine(
            'text', crayon:Green(online)..crayon:Red(offline),
            'wrap',true
         )

    end
 
    local c_loot = Tablet:AddCategory(
        'columns', 3,
        'hasCheck', true,
        'checked', self.db.account.tt.loot,
        'showWithoutChildren', true,
        'text', 'Loot',
        'func', function()
            if self.db.account.tt.loot then
                self.db.account.tt.loot = false
            else
                self.db.account.tt.loot = true
            end
        end,
        'arg1', self
    )
    
    if self.db.account.tt.loot then
         c_loot:AddLine(
            'text', 'Looter',
            'text2', 'Item',
            'text3', 'DKP'     
         )
        for k,v in pairs(self.cr.loot) do 

            local item = self:get_item(v.item)
           
            _,r,g,b = PC:GetRGBPercent("ItemQuality"..item.rarity)
            
            c_loot:AddLine(
                    'text', self:get_player(v.player),
                    'text2', "["..item.name.."]",
                    'text2R',r,'text2G',g,'text2B',b,
                    'justify2', "LEFT",
                    'text3', v.dkp,
                    'func', 'OnTooltipClick',
                    'arg1', self,
                    'arg2', k
            )
        
        end
    end        
    
    local c_zone = Tablet:AddCategory(
        'columns', 1,
        'hasCheck', true,
        'checked', self.db.account.tt.zone,
        'showWithoutChildren', true,
        'text', 'Zones',
        'func', function()
            if self.db.account.tt.zone then
                self.db.account.tt.zone = false
            else
                self.db.account.tt.zone = true
            end
        end,
        'arg1', self    
    )
    
    local c_kills = Tablet:AddCategory(
        'columns', 2,
        'hasCheck', true,
        'checked', self.db.account.tt.kills,
        'showWithoutChildren', true,
        'text', 'Bosskills',
        'func', function()
            if self.db.account.tt.kills then
                self.db.account.tt.kills = false
            else
                self.db.account.tt.kills = true
            end
        end,
        'arg1', self    
    )
    
    if self.db.account.tt.kills then
 
        for k,v in pairs(self.cr.kills) do 

            c_kills:AddLine(
                'text', self:FormatDate(v[1]),
                'text2', self:Id2Boss(v[2])
            )
        
        end
    end        
    
    --Tablet:SetHint("Click to detach Tooltip")
end


function TuiRaidTracker:SetMinRarity(x)
    self.db.account.settings.minrarity = x
end


function TuiRaidTracker:OnMenuRequest(level, value, inTooltip, value_2)
    local r,g,b
    
    if level == 1 then
    
        dewdrop:AddLine(
            'text', "Raids",
            'hasArrow',true,
            'value', "raids"
        )        

        dewdrop:AddLine(
            'text', "Raritylevel",
            'hasArrow',true,
            'value', "rarity",
            'tooltipTitle', "Raritylevel",
            'tooltipText', "Which Items should we log"
        )

        dewdrop:AddLine()
        
        dewdrop:AddLine(
            'text', 'Plugin Settings',
            'isTitle', 'true'
        )
    
    elseif level == 2 then
        if value == "raids" then
            
            for k,v in pairs(TLR.raid) do 
                
                dewdrop:AddLine(
                    'text', string.format("%s %s",self:FormatDate(k),self:Id2Zone(v.lastzone) ),
                    'value',k,
                    'hasArrow',true,
                    'notCheckable', true
                )
            
                self:Debug("raids",v.lastzone, self:FormatDate(k))
            end
            
        
        elseif value == "rarity" then
            _,r,g,b = PC:GetRGBPercent("ItemQuality0")
            dewdrop:AddLine(
                'textR',r,'textG',g,'textB',b,
                'text', "Poor",
                'checked', self.db.account.settings.minrarity == 0,
                'func', 'SetMinRarity',
                'arg1', self,
                'arg2', 0
            )
            _,r,g,b = PC:GetRGBPercent("ItemQuality1")
            dewdrop:AddLine(
                'textR',r,'textG',g,'textB',b,
                'text', "Common",
                'checked', self.db.account.settings.minrarity == 1,
                'func', 'SetMinRarity',
                'arg1', self,
                'arg2', 1
            )
            _,r,g,b = PC:GetRGBPercent("ItemQuality2")
            dewdrop:AddLine(
                'textR',r,'textG',g,'textB',b,
                'text', "Uncommon",
                'checked', self.db.account.settings.minrarity == 2,
                'func', 'SetMinRarity',
                'arg1', self,
                'arg2', 2
            )
            _,r,g,b = PC:GetRGBPercent("ItemQuality3")
            dewdrop:AddLine(
                'textR',r,'textG',g,'textB',b,
                'text', "Rare",
                'checked', self.db.account.settings.minrarity == 3,
                'func', 'SetMinRarity',
                'arg1', self,
                'arg2', 3
            )
            _,r,g,b = PC:GetRGBPercent("ItemQuality4")
            dewdrop:AddLine(
                'textR',r,'textG',g,'textB',b,
                'text', "Epic",
                'checked', self.db.account.settings.minrarity == 4,
                'func', 'SetMinRarity',
                'arg1', self,
                'arg2', 4  
            )
            _,r,g,b = PC:GetRGBPercent("ItemQuality5")
            dewdrop:AddLine(
                'textR',r,'textG',g,'textB',b,
                'text', "Legendary",
                'checked', self.db.account.settings.minrarity == 5,
                'func', 'SetMinRarity',
                'arg1', self,
                'arg2', 5
            )
            _,r,g,b = PC:GetRGBPercent("ItemQuality6")
            dewdrop:AddLine(
                'textR',r,'textG',g,'textB',b,
                'text', "Artifact",
                'checked', self.db.account.settings.minrarity == 6,
                'func', 'SetMinRarity',
                'arg1', self,
                'arg2', 6
            )
        end
    
    elseif level == 3 then
        if value_2 == "raids" then
            self:Debug("level3", value, intooltip, value_2)
            dewdrop:AddLine(
                'text', 'Export',
                'func', 'ExportRaid',
                'arg1', self,
                'arg2', value
            )
            if value ~= TLR.cr then
                dewdrop:AddLine()
                dewdrop:AddLine(
                    'text', 'Remove',
                    'func', function(s,v) TLR.raid[v] = nil; s:Debug("remove",v) end,
                    'arg1', self,
                    'arg2', value
                )
            end
        end
    end
end

-- Binding Variables
BINDING_HEADER_MYTUIRAIDTRACKER = "TuiRaidTracker";
BINDING_NAME_TUIRAIDTRACKER = "Show Window";
